@@ -1,24 +1,23 @@
-#!/usr/bin/env python2
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+#!/usr/bin/env python
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import copy
+from collections import OrderedDict
 from functools import partial
-from future_builtins import map
+from polyglot.builtins import iteritems, native_string_type
 
 from calibre.ebooks.metadata import author_to_author_sort
-from calibre.utils.config_base import tweaks
+from calibre.utils.config_base import tweaks, prefs
 from calibre.utils.icu import sort_key, collation_order
 
 CATEGORY_SORTS = ('name', 'popularity', 'rating')  # This has to be a tuple not a set
 
 
-class Tag(object):
+class Tag:
 
     __slots__ = ('name', 'original_name', 'id', 'count', 'state', 'is_hierarchical',
             'is_editable', 'is_searchable', 'id_set', 'avg_rating', 'sort',
@@ -43,14 +42,16 @@ class Tag(object):
         self.search_expression = search_expression
         self.original_categories = None
 
-    def __unicode__(self):
-        return u'%s:%s:%s:%s:%s'%(self.name, self.count, self.id, self.state, self.category)
+    @property
+    def string_representation(self):
+        return '%s:%s:%s:%s:%s:%s'%(self.name, self.count, self.id, self.state,
+                                    self.category, self.original_categories)
 
     def __str__(self):
-        return unicode(self).encode('utf-8')
+        return self.string_representation
 
     def __repr__(self):
-        return str(self)
+        return native_string_type(self)
 
     __calibre_serializable__ = True
 
@@ -66,7 +67,7 @@ class Tag(object):
 
 
 def find_categories(field_metadata):
-    for category, cat in field_metadata.iteritems():
+    for category, cat in field_metadata.iter_items():
         if (cat['is_category'] and cat['kind'] not in {'user', 'search'}):
             yield (category, cat['is_multiple'].get('cache_to_list', None), False)
         elif (cat['datatype'] == 'composite' and
@@ -101,8 +102,8 @@ def clean_user_categories(dbcache):
         if len(comps) == 0:
             i = 1
             while True:
-                if unicode(i) not in user_cats:
-                    new_cats[unicode(i)] = user_cats[k]
+                if str(i) not in user_cats:
+                    new_cats[str(i)] = user_cats[k]
                     break
                 i += 1
         else:
@@ -115,17 +116,59 @@ def clean_user_categories(dbcache):
     return new_cats
 
 
+def is_standard_category(key):
+    return not (key.startswith('@') or key == 'search')
+
+
+def category_display_order(ordered_cats, all_cats):
+    # ordered_cats is the desired order. all_cats is the list of keys returned
+    # by get_categories, which is in the default order
+    cat_ord = []
+    all_cat_set = frozenset(all_cats)
+    # Do the standard categories first
+    # Verify all the columns in ordered_cats are actually in all_cats
+    for key in ordered_cats:
+        if is_standard_category(key) and key in all_cat_set:
+            cat_ord.append(key)
+    # Add any new standard cats at the end of the list
+    for key in all_cats:
+        if key not in cat_ord and is_standard_category(key):
+            cat_ord.append(key)
+    # Now add the non-standard cats (user cats and search)
+    for key in all_cats:
+        if not is_standard_category(key):
+            cat_ord.append(key)
+    return cat_ord
+
+
+numeric_collation = prefs['numeric_collation']
+
+
+def sort_key_for_name_and_first_letter(x):
+    v1 = icu_upper(x.sort or x.name)
+    v2 = v1 or ' '
+    # The idea is that '9999999999' is larger than any digit so all digits
+    # will sort in front. Non-digits will sort according to their ICU first letter
+    c = v2[0]
+    return (c if numeric_collation and c.isdigit() else '9999999999',
+            collation_order(v2), sort_key(v1))
+
+
 category_sort_keys = {True:{}, False: {}}
 category_sort_keys[True]['popularity'] = category_sort_keys[False]['popularity'] = \
     lambda x:(-getattr(x, 'count', 0), sort_key(x.sort or x.name))
 category_sort_keys[True]['rating'] = category_sort_keys[False]['rating'] = \
     lambda x:(-getattr(x, 'avg_rating', 0.0), sort_key(x.sort or x.name))
 category_sort_keys[True]['name'] = \
-    lambda x:(collation_order(icu_upper(x.sort or x.name or ' ')), sort_key(x.sort or x.name))
+    sort_key_for_name_and_first_letter
 category_sort_keys[False]['name'] = \
     lambda x:sort_key(x.sort or x.name)
 
 
+# Various parts of calibre depend on the the order of fields in the returned
+# dict being in the default display order: standard fields, custom in alpha order,
+# user categories, then saved searches. This works because the backend adds
+# custom columns to field metadata in the right order.
 def get_categories(dbcache, sort='name', book_ids=None, first_letter_sort=False):
     if sort not in CATEGORY_SORTS:
         raise ValueError('sort ' + sort + ' not a valid value')
@@ -134,7 +177,7 @@ def get_categories(dbcache, sort='name', book_ids=None, first_letter_sort=False)
     book_rating_map = dbcache.fields['rating'].book_value_map
     lang_map = dbcache.fields['languages'].book_value_map
 
-    categories = {}
+    categories = OrderedDict()
     book_ids = frozenset(book_ids) if book_ids else book_ids
     pm_cache = {}
 
@@ -194,10 +237,11 @@ def get_categories(dbcache, sort='name', book_ids=None, first_letter_sort=False)
     for c in gst:
         if c not in muc:
             continue
-        user_categories[c] = []
+        uc = []
         for sc in gst[c]:
             for t in categories.get(sc, ()):
-                user_categories[c].append([t.name, sc, 0])
+                uc.append([t.name, sc, 0])
+        user_categories[c] = uc
 
     if user_categories:
         # We want to use same node in the user category as in the source
@@ -206,11 +250,11 @@ def get_categories(dbcache, sort='name', book_ids=None, first_letter_sort=False)
         # do the verification in the category loop much faster, at the cost of
         # temporarily duplicating the categories lists.
         taglist = {}
-        for c, items in categories.iteritems():
+        for c, items in iteritems(categories):
             taglist[c] = dict(map(lambda t:(icu_lower(t.name), t), items))
 
         # Add the category values to the user categories
-        for user_cat in sorted(user_categories.iterkeys(), key=sort_key):
+        for user_cat in sorted(user_categories, key=sort_key):
             items = []
             names_seen = {}
             user_cat_is_gst = user_cat in gst
