@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2013, Kovid Goyal <kovid at kovidgoyal.net>
 
 
@@ -67,7 +66,7 @@ _dff = None
 def default_font_family():
     global _dff
     if _dff is None:
-        families = set(map(str, QFontDatabase().families()))
+        families = set(map(str, QFontDatabase.families()))
         for x in ('Ubuntu Mono', 'Consolas', 'Liberation Mono'):
             if x in families:
                 _dff = x
@@ -187,7 +186,7 @@ class TextEdit(PlainTextEdit):
                 if mt.startswith('image/'):
                     self.insert_image(href)
                 elif mt in OEB_STYLES:
-                    insert_text('<link href="{}" rel="stylesheet" type="text/css"/>'.format(href))
+                    insert_text(f'<link href="{href}" rel="stylesheet" type="text/css"/>')
                 elif mt in OEB_DOCS:
                     self.insert_hyperlink(href, name)
             self.ensureCursorVisible()
@@ -225,10 +224,10 @@ class TextEdit(PlainTextEdit):
         self.setLineWrapMode(QPlainTextEdit.LineWrapMode.WidgetWidth if prefs['editor_line_wrap'] else QPlainTextEdit.LineWrapMode.NoWrap)
         theme = get_theme(prefs['editor_theme'])
         self.apply_theme(theme)
-        w = self.fontMetrics()
-        self.space_width = w.width(' ')
+        fm = self.fontMetrics()
+        self.space_width = fm.horizontalAdvance(' ')
         self.tw = self.smarts.override_tab_stop_width if self.smarts.override_tab_stop_width is not None else prefs['editor_tab_stop_width']
-        self.setTabStopWidth(self.tw * self.space_width)
+        self.setTabStopDistance(self.tw * self.space_width)
         if dictionaries_changed:
             self.highlighter.rehighlight()
 
@@ -254,14 +253,14 @@ class TextEdit(PlainTextEdit):
         if ff is None:
             ff = default_font_family()
         font.setFamily(ff)
-        font.setPointSize(tprefs['editor_font_size'])
+        font.setPointSizeF(tprefs['editor_font_size'])
         self.tooltip_font = QFont(font)
-        self.tooltip_font.setPointSize(font.pointSize() - 1)
+        self.tooltip_font.setPointSizeF(font.pointSizeF() - 1.)
         self.setFont(font)
         self.highlighter.apply_theme(theme)
-        w = self.fontMetrics()
-        self.number_width = max(map(lambda x:w.width(str(x)), range(10)))
-        self.size_hint = QSize(self.expected_geometry[0] * w.averageCharWidth(), self.expected_geometry[1] * w.height())
+        fm = self.fontMetrics()
+        self.number_width = max(map(lambda x:fm.horizontalAdvance(str(x)), range(10)))
+        self.size_hint = QSize(self.expected_geometry[0] * fm.averageCharWidth(), self.expected_geometry[1] * fm.height())
         self.highlight_color = theme_color(theme, 'HighlightRegion', 'bg')
         self.highlight_cursor_line()
         self.completion_popup.clear_caches(), self.completion_popup.update()
@@ -277,7 +276,7 @@ class TextEdit(PlainTextEdit):
             self.smarts = sclass(self)
             if self.smarts.override_tab_stop_width is not None:
                 self.tw = self.smarts.override_tab_stop_width
-                self.setTabStopWidth(self.tw * self.space_width)
+                self.setTabStopDistance(self.tw * self.space_width)
         if isinstance(text, bytes):
             text = text.decode('utf-8', 'replace')
         self.setPlainText(unicodedata.normalize('NFC', str(text)))
@@ -567,7 +566,7 @@ class TextEdit(PlainTextEdit):
             c.movePosition(QTextCursor.MoveOperation.Start)
         block = c.block()
         while block.isValid():
-            for r in block.layout().additionalFormats():
+            for r in block.layout().formats():
                 if r.format.property(SPELL_PROPERTY):
                     if not from_cursor or block.position() + r.start + r.length > c.position():
                         c.setPosition(block.position() + r.start)
@@ -731,7 +730,7 @@ class TextEdit(PlainTextEdit):
         c.movePosition(QTextCursor.MoveOperation.Start)
         block = c.block()
         while block.isValid():
-            for r in block.layout().additionalFormats():
+            for r in block.layout().formats():
                 if r.format.property(SPELL_PROPERTY) and self.text_for_range(block, r) == word:
                     self.highlighter.reformat_block(block)
                     break
@@ -742,7 +741,7 @@ class TextEdit(PlainTextEdit):
         if cursor.isNull():
             return
         pos = cursor.positionInBlock()
-        for r in cursor.block().layout().additionalFormats():
+        for r in cursor.block().layout().formats():
             if r.start <= pos <= r.start + r.length and r.format.property(SYNTAX_PROPERTY):
                 return r
 
@@ -811,7 +810,7 @@ class TextEdit(PlainTextEdit):
             # For some reason using eventFilter for this does not work, so we
             # implement it here
             self.completion_popup.abort()
-        if ev.modifiers() & Qt.Modifier.CTRL:
+        if ev.modifiers() & Qt.KeyboardModifier.ControlModifier:
             url = self.link_for_position(ev.pos())
             if url is not None:
                 ev.accept()
@@ -866,25 +865,13 @@ class TextEdit(PlainTextEdit):
             'bold': ('<b>', '</b>'),
             'italic': ('<i>', '</i>'),
             'underline': ('<u>', '</u>'),
-            'strikethrough': ('<strike>', '</strike>'),
+            'strikethrough': ('<s>', '</s>'),
             'superscript': ('<sup>', '</sup>'),
             'subscript': ('<sub>', '</sub>'),
             'color': ('<span style="color: %s">' % color, '</span>'),
             'background-color': ('<span style="background-color: %s">' % color, '</span>'),
         }[formatting]
-        left, right = self.get_range_inside_tag()
-        c = self.textCursor()
-        c.setPosition(left)
-        c.setPosition(right, QTextCursor.MoveMode.KeepAnchor)
-        prev_text = str(c.selectedText()).rstrip('\0')
-        c.insertText(prefix + prev_text + suffix)
-        if prev_text:
-            right = c.position()
-            c.setPosition(left)
-            c.setPosition(right, QTextCursor.MoveMode.KeepAnchor)
-        else:
-            c.setPosition(c.position() - len(suffix))
-        self.setTextCursor(c)
+        self.smarts.surround_with_custom_tag(self, prefix, suffix)
 
     def insert_image(self, href, fullpage=False, preserve_aspect_ratio=False, width=-1, height=-1):
         if width <= 0:
@@ -908,7 +895,7 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 {w} {h}" preserveAspectR
 </svg></div>'''.format(w=width, h=height, a='xMidYMid meet' if preserve_aspect_ratio else 'none')
             else:
                 alt = _('Image')
-                template = '<img alt="{}" src="%s" />'.format(alt)
+                template = f'<img alt="{alt}" src="%s" />'
         text = template % href
         c.insertText(text)
         if self.syntax == 'html' and not fullpage:
